# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import output_ascii_table, dash_if_none
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def output_account_info(output_media=None, account_id=None, account_aliases=None):
    """
    @type output_media: unicode
    @type account_id: unicode
    @type account_aliases: list
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}id{/autoblue}'), dash_if_none(account_id)])
        td.append([Color('{autoblue}aliases{/autoblue}'), dash_if_none(", ".join(account_aliases))])
        output_ascii_table(table_title=Color('{autowhite}Account Info{/autowhite}'),
                           table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import output_ascii_table, dash_if_none, get_tags
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def get_instances_output(instances):
    """
    @type instances: list
    """
    ret = ""
    for instance in instances:
        ret += "{0}\n".format(instance.get('InstanceId'))
    if ret:
        return ret.rstrip()


def output_asg_list(output_media=None, asg_list=None):
    """
    @type output_media: unicode
    @type asg_list: list
    """
    if isinstance(asg_list, list):
        if output_media == 'console':
            td = [[Color('{autoblue}name{/autoblue}'), Color('{autoblue}instances{/autoblue}'),
                   Color('{autoblue}min{/autoblue}'), Color('{autoblue}max{/autoblue}'),
                   Color('{autoblue}lc{/autoblue}')]]
            for asg in asg_list:
                td.append([asg.get('AutoScalingGroupName', '-'),
                           str(len(asg.get('Instances', '-'))),
                           str(asg.get('MinSize', '-')),
                           str(asg.get('MaxSize', '-')),
                           asg.get('LaunchConfigurationName', '-')
                           ])
            output_ascii_table(table_title=Color('{autowhite}ASGs{/autowhite}'),
                               table_data=td,
                               inner_heading_row_border=True)
    exit(0)


def output_asg_info(output_media=None, asg=None):
    """
    @type output_media: unicode
    @type asg: dict
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}name{/autoblue}'), dash_if_none(asg.get('AutoScalingGroupName'))])
        #td.append([Color('{autoblue}arn{/autoblue}'), dash_if_none(asg.get('AutoScalingGroupARN'))])
        td.append([Color('{autoblue}lc{/autoblue}'), dash_if_none(asg.get('LaunchConfigurationName'))])
        td.append([Color('{autoblue}min size{/autoblue}'), str(dash_if_none(asg.get('MinSize')))])
        td.append([Color('{autoblue}max size{/autoblue}'), str(dash_if_none(asg.get('MaxSize')))])
        td.append([Color('{autoblue}desired size{/autoblue}'), str(dash_if_none(asg.get('DesiredCapacity')))])
        td.append([Color('{autoblue}default cooldown{/autoblue}'), str(dash_if_none(asg.get('DefaultCooldown')))])
        td.append([Color('{autoblue}availability zones{/autoblue}'), str(dash_if_none('\n'.join(asg.get('AvailabilityZones'))))])
        td.append([Color('{autoblue}load balancer names{/autoblue}'), str(dash_if_none('\n'.join(asg.get('LoadBalancerNames'))))])
        td.append([Color('{autoblue}health check type{/autoblue}'), str(dash_if_none(asg.get('HealthCheckType')))])
        td.append([Color('{autoblue}health check grace period{/autoblue}'), str(dash_if_none(asg.get('HealthCheckGracePeriod')))])
        td.append([Color('{autoblue}instances{/autoblue}'), dash_if_none(get_instances_output(asg.get('Instances')))])
        td.append([Color('{autoblue}created{/autoblue}'), str(dash_if_none(asg.get('CreatedTime').replace(tzinfo=None, microsecond=0)))])
        td.append([Color('{autoblue}suspended processes{/autoblue}'), dash_if_none(asg.get('SuspendedProcesses'))])
        td.append([Color('{autoblue}placement group{/autoblue}'), dash_if_none(asg.get('PlacementGroup'))])
        td.append([Color('{autoblue}vpc zone identifier{/autoblue}'), str(dash_if_none(asg.get('VPCZoneIdentifier')))])
        td.append([Color('{autoblue}metrics enabled{/autoblue}'), dash_if_none(asg.get('EnabledMetrics'))])
        td.append([Color('{autoblue}status{/autoblue}'), dash_if_none(asg.get('Status'))])
        td.append([Color('{autoblue}tags{/autoblue}'), dash_if_none(get_tags(asg.get('Tags')))])
        td.append([Color('{autoblue}termination policies{/autoblue}'), str(dash_if_none(asg.get('TerminationPolicies')))])
        output_ascii_table(table_title=Color('{autowhite}asg info{/autowhite}'),
                           table_data=td)
    exit(0)


def output_lc_list(output_media=None, lc_list=None):
    """
    @type output_media: unicode
    @type lc_list: list
    """
    if isinstance(lc_list, list):
        if output_media == 'console':
            td = [[Color('{autoblue}name{/autoblue}'), Color('{autoblue}image id{/autoblue}'),
                   Color('{autoblue}instance type{/autoblue}'), Color('{autoblue}created{/autoblue}')]]
            for lc in lc_list:
                td.append([lc.get('LaunchConfigurationName', '-'),
                           str(lc.get('ImageId', '-')),
                           str(lc.get('InstanceType', '-')),
                           str(lc.get('CreatedTime').replace(tzinfo=None, microsecond=0))
                           ])
            output_ascii_table(table_title=Color('{autowhite}launch configurations{/autowhite}'),
                               table_data=td,
                               inner_heading_row_border=True)
    exit(0)


def output_lc_info(output_media=None, lc=None):
    """
    @type output_media: unicode
    @type lc: dict
    """
    pass
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}name{/autoblue}'), dash_if_none(lc.get('LaunchConfigurationName'))])
        td.append([Color('{autoblue}image id{/autoblue}'), dash_if_none(lc.get('ImageId'))])
        td.append([Color('{autoblue}instance type{/autoblue}'), dash_if_none(lc.get('InstanceType'))])
        td.append([Color('{autoblue}created (UTC){/autoblue}'), str(dash_if_none(lc.get('CreatedTime').replace(tzinfo=None, microsecond=0)))])
        td.append([Color('{autoblue}iam instance profile{/autoblue}'), dash_if_none(lc.get('IamInstanceProfile'))])
        td.append([Color('{autoblue}ebs optimised{/autoblue}'), dash_if_none("Yes" if lc.get('EbsOptimized') else "No")])
        td.append([Color('{autoblue}instance monitoring{/autoblue}'), dash_if_none("Yes" if lc.get('InstanceMonitoring').get('Enabled') else "No" )])
        td.append([Color('{autoblue}classiclink VPC sec groups{/autoblue}'), dash_if_none(lc.get('ClassicLinkVPCSecurityGroups'))])
        td.append([Color('{autoblue}block device mappings{/autoblue}'), dash_if_none(lc.get('BlockDeviceMappings'))])
        td.append([Color('{autoblue}keypair{/autoblue}'), dash_if_none(lc.get('KeyName'))])
        td.append([Color('{autoblue}security groups{/autoblue}'), dash_if_none(", ".join(lc.get('SecurityGroups')))])
        td.append([Color('{autoblue}kernel ID{/autoblue}'), dash_if_none(lc.get('KernelId'))])
        td.append([Color('{autoblue}ramdisk id{/autoblue}'), dash_if_none(lc.get('RamdiskId'))])
        td.append([Color('{autoblue}image id{/autoblue}'), dash_if_none(lc.get('ImageId'))])
        td.append([Color('{autoblue}instance typr{/autoblue}'), dash_if_none(lc.get('InstanceType'))])
        td.append([Color('{autoblue}associate public ip{/autoblue}'), dash_if_none(lc.get('AssociatePublicIpAddress'))])
        td.append([Color('{autoblue}user data{/autoblue}'), dash_if_none(lc.get('UserData'))])
        output_ascii_table(table_title=Color('{autowhite}launch configuration info{/autowhite}'),
                           table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
"""Usage:
    acli lc (list | info <lc_name>) [options]

    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from terminaltables import AsciiTable
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def output_ascii_table(table_title=None,
                       table_data=None,
                       inner_heading_row_border=False,
                       inner_row_border=False):
    """
    @type table_title: unicode
    @type table_data: list
    @type inner_heading_row_border: bool
    @type inner_row_border: bool
    """
    table = AsciiTable(table_data)
    table.inner_heading_row_border = inner_heading_row_border
    table.inner_row_border = inner_row_border
    table.title = table_title
    print(table.table)


def dash_if_none(item=None):
    """
    @type item: object
    """
    return str(item) if item else Color('{autoblack}-{/autoblack}')


def get_tags(tags, separator=', '):
    """
    @type tags: list
    @type separator: unicode
    """
    tag_list = list()
    for tag in tags:
        tag_list.append("{0}:{1}".format(tag.get('Key'), tag.get('Value')))
    if tag_list:
        return separator.join(tag_list)


def get_name_tag(tags):
    """
    @type tags: dict
    """
    for tag_name, tag_value in tags.iteritems():
        if tag_name == 'Name':
            return tag_value
    return "-"



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.eip import (output_eip_list, output_eip_info)
from acli.connections import get_client
from botocore.exceptions import ClientError


def eip_list(aws_config=None):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    addresses_response = ec2_client.describe_addresses()
    addresses = addresses_response.get('Addresses', None)
    if len(list(addresses)):
        output_eip_list(output_media='console', addresses=addresses)
    exit('No elastic IPs found.')


def eip_info(aws_config=None, eip=None):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    try:
        addresses_response = ec2_client.describe_addresses(PublicIps=[eip])
        address = addresses_response.get('Addresses', None)[0]
        if len(list(address)):
            output_eip_info(output_media='console', address=address)
    except ClientError:
        exit('EIP {0} not found.'.format(eip))



# -*- coding: utf-8 -*-
"""Usage:
    acli ec2 (list | summary) [options] [--region=<region>]
    acli ec2 (start | stop | reboot | terminate | info | cpu | vols | net) <instance_id> [options]

    -s, --start=<start_date>        metrics start-date
    -e, --end=<end_date>            metrics end-date
    -p, --period=<period>            metrics period
    -i, --intervals=<intervals>     metrics intervals
    -o, --output=<output_type>      table, json, yaml or graph [default: table].
    -h, --help
"""
from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt

if __name__ == '__main__':
    print(docopt(__doc__))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, dash_if_none)
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def colour_state(state=None):
    if not state:
        return Color('{autoblack}-{/autoblack}')
    elif state == 'running':
        return Color('{autogreen}'+state+'{/autogreen}')
    elif state in ('stopped', 'stopping', 'shutting-down', 'terminated'):
        return Color('{autored}'+state+'{/autored}')
    elif state in ('rebooting', 'pending'):
        return Color('{autoyellow}'+state+'{/autoyellow}')


def output_eip_list(output_media=None, addresses=None):
    """
    @type output_media: unicode
    @type addresses: list
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}public ip{/autoblue}'),
                   Color('{autoblue}instance id{/autoblue}'),
                   Color('{autoblue}domain{/autoblue}'),
                   Color('{autoblue}private ip address{/autoblue}')])
        for addr in addresses:
            td.append([dash_if_none(addr.get('PublicIp', None)),
                       dash_if_none(addr.get('InstanceId')),
                       dash_if_none(addr.get('Domain', None)),
                       dash_if_none(addr.get('PrivateIpAddress', None))])
        output_ascii_table(table_title=Color('{autowhite}address list{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    exit(0)


def output_eip_info(output_media=None, address=None):
    """
    @type output_media: unicode
    @type address: dict
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}public ip{/autoblue}'),
                   dash_if_none(address.get('PublicIp', None))])
        td.append([Color('{autoblue}allocation id{/autoblue}'),
                   dash_if_none(address.get('AllocationId', None))])
        td.append([Color('{autoblue}instance id{/autoblue}'),
                   dash_if_none(address.get('InstanceId', None))])
        td.append([Color('{autoblue}association id{/autoblue}'),
                   dash_if_none(address.get('AssociationId', None))])
        td.append([Color('{autoblue}domain{/autoblue}'),
                   dash_if_none(address.get('Domain', None))])
        td.append([Color('{autoblue}network interface id{/autoblue}'),
                   dash_if_none(address.get('NetworkInterfaceId', None))])
        td.append([Color('{autoblue}network interface owner id{/autoblue}'),
                   dash_if_none(address.get('NetworkInterfaceOwnerId', None))])
        td.append([Color('{autoblue}private ip{/autoblue}'),
                   dash_if_none(address.get('PrivateIpAddress', None))])
        output_ascii_table(table_title=Color('{autowhite}eip info{/autowhite}'),
                           table_data=td)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table)
import re
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def output_s3_list(output_media=None, buckets=None, bucket_name=None, objects=None, folders=None, item=None):
    """
    @type output_media: unicode
    @type buckets: dict
    @type objects: dict
    @type bucket_name: unicode
    """
    to_remove_len = 0
    if bucket_name:
        to_remove_len = len(re.sub(bucket_name, '', item))-1
    if buckets:
        sorted_buckets = sorted(buckets, key=lambda k: ['Name'])
        td = list()
        td.append([Color('{autoblue}name{/autoblue}'), Color('{autoblue}created{/autoblue}')])
        for bucket in sorted_buckets:
            if output_media == 'console':
                td.append([bucket.get('Name'),
                           str(bucket.get('CreationDate').replace(tzinfo=None, microsecond=0))])
        output_ascii_table(table_title=Color('{autowhite}s3 buckets{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    if any((objects, folders)):
        td = list()
        td.append([Color('{autoblue}item{/autoblue}'), Color('{autoblue}size (bytes){/autoblue}'),
                  Color('{autoblue}last modified (UTC){/autoblue}'),
                  Color('{autoblue}class{/autoblue}'), Color('{autoblue}etag{/autoblue}')])
        if folders:
            for folder in folders:
                td.append([Color('{autogreen}'+folder.get('Prefix')[to_remove_len:]+'{/autogreen}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}'),
                           Color('{autoblack}-{/autoblack}')])
        object_list = objects.get('Contents', None)
        if objects.get('Contents', None):
            sorted_object_list = sorted(object_list, key=lambda k: ['Key'])
            for index, an_object in enumerate(sorted_object_list):
                an_object_key = an_object.get('Key')[to_remove_len:]
                if an_object_key:
                    td.append([an_object_key,
                               str(an_object.get('Size')),
                               str(an_object.get('LastModified').replace(tzinfo=None, microsecond=0)),
                               str(an_object.get('StorageClass')),
                               # str(an_object.get('Owner')),
                               str(an_object.get('ETag')[1:-1])])
        output_ascii_table(table_title=Color('{autowhite}'+item+'{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    exit(0)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, dash_if_none)
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def get_tag(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')


def split_string(the_string=None, chunk_size=None):
    length = 0
    output = str()
    chunks = the_string.split()
    for chunk in chunks:
        length += len(chunk)
        output += chunk
        if length >= chunk_size:
            output += "\n"
            length = 0
        else:
            output += " "
    return output.rstrip()


def output_secgroup_list(output_media=None, secgroups=None):
    """
    @type output_media: unicode
    @type secgroups: dict
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}group id{/autoblue}'),
                   Color('{autoblue}group name{/autoblue}'),
                   Color('{autoblue}description{/autoblue}')])
        for secgroup in secgroups.get('SecurityGroups'):
            td.append([secgroup.get('GroupId'),
                      split_string(secgroup.get('GroupName'), chunk_size=30),
                      split_string(secgroup.get('Description'), chunk_size=40)])
        output_ascii_table(table_title=Color('{autowhite}security groups{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    exit(0)


def get_ip_ranges(ip_ranges):
    out = str()
    for ip_range in ip_ranges:
        out += "{0}\n".format(ip_range.get('CidrIp'))
    return out.rstrip()


def get_uid_group_pairs(uid_gps=None):
    out = str()
    for uid_gp in uid_gps:
        out += '{0} - {1}\n'.format(uid_gp.get('UserId'), uid_gp.get('GroupId'))
    return out.strip()


def output_secgroup_info(output_media=None, secgroup=None):
    """
    @type output_media: unicode
    @type secgroup: ec2.SecurityGroup
    """
    if secgroup:
        if output_media == 'console':
            td = list()
            td.append([Color('{autoblue}group id{/autoblue}'), secgroup.get('GroupId')])
            td.append([Color('{autoblue}group name{/autoblue}'), secgroup.get('GroupName')])
            td.append([Color('{autoblue}description{/autoblue}'), secgroup.get('Description')])
            td.append([Color('{autoblue}vpc id{/autoblue}'), dash_if_none(secgroup.get('VpcId'))])
            if secgroup.get('IpPermissions'):
                td.append(['ip permissions', '{0}'.format("-" * 30)])
                for ip_perm in secgroup.get('IpPermissions'):
                    td.append([Color('{autoblue}from port{/autoblue}'),
                               dash_if_none(ip_perm.get('FromPort'))])
                    td.append([Color('{autoblue}to port{/autoblue}'),
                               dash_if_none(ip_perm.get('ToPort'))])
                    td.append([Color('{autoblue}ip ranges{/autoblue}'),
                               dash_if_none(get_ip_ranges(ip_perm.get('IpRanges')))])
                    td.append([Color('{autoblue}ip protocol{/autoblue}'),
                               dash_if_none(ip_perm.get('IpProtocol'))])
                    if ip_perm.get('UserIdGroupPairs'):
                        td.append(['user id group pairs', '{0}'.format("-" * 30)])
                        td.append(['', get_uid_group_pairs(uid_gps=ip_perm.get('UserIdGroupPairs'))])

            if secgroup.get('IpPermissionsEgress'):
                td.append(['ip permissions egress', '{0}'.format("-" * 30)])
                for ip_perm_egress in secgroup.get('IpPermissionsEgress'):
                    td.append([Color('{autoblue}prefix list ids{/autoblue}'),
                               dash_if_none(ip_perm_egress.get('PrefixListIds'))])
                    td.append([Color('{autoblue}from port{/autoblue}'),
                               dash_if_none(ip_perm_egress.get('FromPort'))])
                    td.append([Color('{autoblue}to port{/autoblue}'),
                               dash_if_none(ip_perm_egress.get('ToPort'))])
                    td.append([Color('{autoblue}ip ranges{/autoblue}'),
                               dash_if_none(get_ip_ranges(ip_perm_egress.get('IpRanges')))])
                    td.append([Color('{autoblue}ip protocol{/autoblue}'),
                               dash_if_none(ip_perm_egress.get('IpProtocol'))])
                    if ip_perm_egress.get('UserIdGroupPairs'):
                        td.append(['user id group pairs', '{0}'.format("-" * 30)])
                        td.append(['', get_uid_group_pairs(uid_gps=ip_perm_egress.get('UserIdGroupPairs'))])
        output_ascii_table(table_title=Color('{autowhite}security group info{/autowhite}'),
                           table_data=td)
    else:
        exit('Security group does not exist.')
    exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
usage: acli [--version] [--help] [--install-completion]
       acli [--region=<region>] [--access_key_id=<access_key_id>] [--secret_access_key=<secret_access_key>]
            <command> [<args>...]
       acli account [options]
       acli ec2 (list | summary) [options]
       acli ec2 (start | stop | reboot | terminate | info | cpu | vols | net) <instance_id> [options]
       acli lc list [options]
       acli asg list [options]
       acli asg (info | cpu | mem | net | delete) <asg_name> [options]
       acli ami (list | info <ami_id>)
       acli eip (list | info <eip>)
       acli elb (list | info <elb_name>)
       acli route53 (list | info <zone_id>)
       acli vpc (list | info <vpc_id>)
       acli secgroup (list | info)
       acli s3 list [<item>]
       acli s3 info <item>

options:
   -h, --help      help

The most common commands are:
   account      Get account info
   ec2          Manage ec2 instances
   eip          Manage elastic IPs
   elb          Manage elb instances
   ami          Manage amis
   asg          Manage auto-scaling groups
   lc           Manage launch configurations
   eip          Manage elastic ips
   secgroup     Manage security groups
   route53      Manage route 53 configuration
   vpc          Manage VPCs
   secgroup     Manage Security Groups

See 'acli help <command>'
"""

from __future__ import (absolute_import, print_function, unicode_literals)
from docopt import docopt
from acli.services import (ec2, elb, account, cloudwatch, vpc,
                           eip, asg, route53, secgroup, s3)
from acli.config import Config
from acli import utils


def real_main():
    args = docopt(__doc__,
                  version='0.1.6',
                  options_first=True)
    aws_config = Config(args)
    argv = [args['<command>']] + args['<args>']
    if args['--install-completion']:
        utils.install_completion()
    if args['<command>'] == 'account':
        from acli.commands import account as command_account
        # acc_res = docopt(command_account.__doc__, argv=argv)
        account.account_info(aws_config)
    if args['<command>'] == 'ec2':
        from acli.commands import ec2 as command_ec2
        ec2_res = docopt(command_ec2.__doc__, argv=argv)
        if ec2_res.get('list'):
            ec2.ec2_list(aws_config)
        elif ec2_res.get('info'):
            ec2.ec2_info(aws_config, instance_id=ec2_res.get('<instance_id>'))
        elif ec2_res.get('stop'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="stop")
        elif ec2_res.get('reboot'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="reboot")
        elif ec2_res.get('start'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="start")
        elif ec2_res.get('terminate'):
            ec2.ec2_manage(aws_config, instance_id=ec2_res.get('<instance_id>'), action="terminate")
        elif ec2_res.get('cpu'):
            cloudwatch.ec2_cpu(aws_config=aws_config, instance_id=ec2_res.get('<instance_id>'))
        elif ec2_res.get('net'):
            cloudwatch.ec2_net(aws_config=aws_config,
                               instance_id=ec2_res.get('<instance_id>'),
                               output_type=ec2_res.get('--output', None),
                               start=ec2_res.get('--start', None),
                               period=ec2_res.get('--end', None),
                               intervals=ec2_res.get('intervals', None)
                               )
        elif ec2_res.get('vols'):
            cloudwatch.ec2_vol(aws_config=aws_config,
                               instance_id=ec2_res.get('<instance_id>'),
                               output_type=ec2_res.get('--output', None),
                               start=ec2_res.get('--start', None),
                               period=ec2_res.get('--end', None),
                               intervals=ec2_res.get('intervals', None)
                               )
        elif ec2_res.get('summary'):
            ec2.ec2_summary(aws_config=aws_config)
    if args['<command>'] == 'elb':
        from acli.commands import elb as command_elb
        elb_res = docopt(command_elb.__doc__, argv=argv)
        if elb_res.get('list'):
            elb.elb_list(aws_config)
        elif elb_res.get('info'):
            elb.elb_info(aws_config, elb_name=elb_res.get('<elb_name>'))
    if args['<command>'] == 'lc':
        from acli.commands import lc as command_lc
        lc_res = docopt(command_lc.__doc__, argv=argv)
        if lc_res.get('list'):
            asg.lc_list(aws_config)
        elif lc_res.get('info'):
            asg.lc_info(aws_config, lc_name=lc_res.get('<lc_name>'))
    if args['<command>'] == 'asg':
        from acli.commands import asg as command_asg
        asg_res = docopt(command_asg.__doc__, argv=argv)
        if asg_res.get('list'):
            asg.asg_list(aws_config)
        elif asg_res.get('info'):
            asg.asg_info(aws_config, asg_name=asg_res.get('<asg_name>'))
        elif asg_res.get('cpu'):
            cloudwatch.asg_cpu(aws_config=aws_config,
                               asg_name=asg_res.get('<asg_name>'),
                               output_type=asg_res.get('--output', None),
                               start=asg_res.get('--start'),
                               period=asg_res.get('--end'),
                               intervals=asg_res.get('intervals')
                               )
        elif asg_res.get('delete'):
            asg.asg_delete(aws_config, asg_name=asg_res.get('<asg_name>'))
    if args['<command>'] == 'ami':
        from acli.commands import ami as command_ami
        ami_res = docopt(command_ami.__doc__, argv=argv)
        if ami_res.get('list'):
            ec2.ami_list(aws_config)
        elif ami_res.get('info'):
            ec2.ami_info(aws_config, ami_id=ami_res.get('<ami_id>'))
    if args['<command>'] == 'route53':
        from acli.commands import route53 as command_route53
        route53_res = docopt(command_route53.__doc__, argv=argv)
        if route53_res.get('list'):
            route53.route53_list(aws_config)
        elif route53_res.get('info'):
            route53.route53_info(aws_config, zone_id=route53_res.get('<zone_id>'))
    if args['<command>'] == 'vpc':
        from acli.commands import vpc as command_vpc
        vpc_res = docopt(command_vpc.__doc__, argv=argv)
        if vpc_res.get('list'):
            vpc.vpc_list(aws_config)
        elif vpc_res.get('info'):
            vpc.vpc_info(aws_config, vpc_id=vpc_res.get('<vpc_id>'))
    if args['<command>'] == 'secgroup':
        from acli.commands import secgroup as command_secgroup
        secgroup_res = docopt(command_secgroup.__doc__, argv=argv)
        if secgroup_res.get('list'):
            secgroup.secgroup_list(aws_config)
        elif secgroup_res.get('info'):
            secgroup.secgroup_info(aws_config, secgroup_id=secgroup_res.get('<secgroup_id>'))
    if args['<command>'] == 'eip':
        from acli.commands import eip as command_eip
        eip_res = docopt(command_eip.__doc__, argv=argv)
        if eip_res.get('list'):
            eip.eip_list(aws_config)
        elif eip_res.get('info'):
            eip.eip_info(aws_config, eip=eip_res.get('<eip>'))
    if args['<command>'] == 's3':
        from acli.commands import s3 as command_s3
        s3_res = docopt(command_s3.__doc__, argv=argv)
        if s3_res.get('list'):
            path = s3_res.get('<item>')
            if path and not '/' == path[-1:]:
                path += '/'
            s3.s3_list(aws_config, item=path)
        elif s3_res.get('info'):
            s3.s3_info(aws_config, item=s3_res.get('<item>'))
    elif args['<command>'] in ['help', None] and args['<args>']:
        if args['<args>'][0] == 'ec2':
            from acli.commands import ec2 as command_ec2
            print(docopt(command_ec2.__doc__, argv=argv))
    elif args['<command>'] in ['help', None] and not args['<args>']:
        print("usage: acli help <command>")
    else:
        exit("%r is not an acli command. See 'acli --help'." % args['<command>'])



from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.ec2 import (output_ec2_list, output_ec2_info)
from acli.services.ec2 import (ec2_list, ec2_info, ec2_summary, ami_list, ami_info)
from acli.services.secgroup import (secgroup_list, secgroup_info)
from acli.config import Config
from moto import mock_ec2
import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


@pytest.yield_fixture(scope='function')
def ec2_instances():
    """EC2 mock service"""
    mock = mock_ec2()
    mock.start()
    client = session.client('ec2')
    client.create_security_group(GroupName='group1', Description='my first sec group')
    instances = client.run_instances(ImageId='ami-12345', MinCount=2, MaxCount=2, SecurityGroups=['group1']).get('Instances')
    for i, s in enumerate(instances):
        client.create_tags(
            Resources=[s.get('InstanceId')],
            Tags=[{'Key': 'Name', 'Value': 'Bob'}])
    yield instances
    mock.stop()


@pytest.yield_fixture(scope='function')
def amis():
    """AMI mock service"""
    mock = mock_ec2()
    mock.start()
    client = session.client('ec2')
    reservation = client.run_instances(ImageId='ami-1234abcd', MinCount=1, MaxCount=1)
    instance = reservation.get('Instances')[0]
    image_id = client.create_image(InstanceId=instance.get('InstanceId'),
                                   Name="test-ami",
                                   Description="this is a test ami")
    yield client.describe_images()
    mock.stop()


@pytest.yield_fixture(scope='function')
def sec_groups():
    """ Mock security groups """
    mock = mock_ec2()
    mock.start()
    client = session.client('ec2')
    client.create_security_group(GroupName='test', Description='test')
    yield client.describe_security_groups()
    mock.stop()


@pytest.yield_fixture(scope='function')
def fake_empty_sec_groups():
    """ Mock security groups """
    mock = mock_ec2()
    mock.start()
    yield None
    mock.stop()


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


def test_ec2_list_service(ec2_instances):
    with pytest.raises(SystemExit):
        assert ec2_list(aws_config=config)


@mock_ec2
def test_ec2_list_service_no_instances():
    with pytest.raises(SystemExit):
        assert ec2_list(aws_config=config)


def test_ec2_info_service(ec2_instances):
    with pytest.raises(SystemExit):
        assert ec2_info(aws_config=config, instance_id=ec2_instances[0].get('InstanceId'))


def test_ec2_list_output(ec2_instances):
    with pytest.raises(SystemExit):
        print(ec2_instances)
        assert output_ec2_list(output_media='console', instances=ec2_instances)


def test_ec2_output(ec2_instances):
    with pytest.raises(SystemExit):
        instance = list(ec2_instances)[0]
        assert output_ec2_info(output_media='console', instance=instance)


def test_ami_list_service(amis):
    with pytest.raises(SystemExit):
        assert ami_list(aws_config=config)


def test_ami_info_service(amis):
    with pytest.raises(SystemExit):
        first_ami_id = amis.get('Images')[0].get('ImageId')
        assert ami_info(aws_config=config, ami_id=first_ami_id)


def test_sec_group_list_service(sec_groups):
    with pytest.raises(SystemExit):
        secgroup_list(aws_config=config)


def test_sec_group_info_service(sec_groups):
    with pytest.raises(SystemExit):
        secgroup_id = sec_groups.get('SecurityGroups')[0].get('GroupId')
        secgroup_info(aws_config=config, secgroup_id=secgroup_id)


def test_sec_group_info_service_with_invalid_id(fake_empty_sec_groups, capsys):
    with pytest.raises(SystemExit):
        invalid_id = 'invalid'
        out, err = capsys.readouterr(secgroup_info(aws_config=config, secgroup_id=invalid_id))
        print(out, err)
        assert err == "Cannot find security group: {0}".format(invalid_id)


# def test_ec2_summary(ec2_instances):
#    with pytest.raises(SystemExit):
#        instance = list(ec2_instances)[0]
#        assert ec2_summary(aws_config=config)



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.secgroup import (output_secgroup_list, output_secgroup_info)
from acli.connections import get_client
from botocore.exceptions import ClientError


def secgroup_list(aws_config=None):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    secgroups = ec2_client.describe_security_groups()
    output_secgroup_list(output_media='console', secgroups=secgroups)


def secgroup_info(aws_config=None, secgroup_id=None):
    """
    @type aws_config: Config
    @type secgroup_id: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    try:
        result = ec2_client.describe_security_groups(GroupIds=[secgroup_id])
        secgroups = result.get('SecurityGroups', None)
        secgroup = secgroups[0]
        output_secgroup_info(output_media='console', secgroup=secgroup)
    except (ClientError, IndexError):
        exit("Cannot find security group: {0}.".format(secgroup_id))



from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.elb import (output_elb_info, output_elbs)
from acli.services.elb import (elb_info, elb_list)
from acli.config import Config
from moto import mock_elb
import pytest
from boto3.session import Session
session = Session(region_name="eu-west-1")


@pytest.yield_fixture(scope='function')
def elb_instances():
    """ELB mock service"""
    mock = mock_elb()
    mock.start()
    client = session.client('elb')
    zones = ['eu-west-1a', 'eu-west-1b']
    listeners = [{
            'Protocol': 'string',
            'LoadBalancerPort': 123,
            'InstanceProtocol': 'string',
            'InstancePort': 123,
            'SSLCertificateId': 'string'}]
    client.create_load_balancer(LoadBalancerName='my-lb', AvailabilityZones=zones, Listeners=listeners)
    yield client.describe_load_balancers()
    mock.stop()


config = Config(cli_args={'--region': 'eu-west-1',
                          '--access_key_id': 'AKIAIOSFODNN7EXAMPLE',
                          '--secret_access_key': 'wJalrXUtnFEMI/K7MDENG/bPxRfiCYEXAMPLEKEY'})


def test_elb_list_service(elb_instances):
    with pytest.raises(SystemExit):
        assert elb_list(aws_config=config)


def test_elb_info_service(elb_instances):
    with pytest.raises(SystemExit):
        assert elb_info(aws_config=config, elb_name='my-lb')



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output.vpc import (output_vpc_list, output_vpc_info)
from acli.connections import get_client
from botocore.exceptions import ClientError


def vpc_list(aws_config=None):
    """
    @type aws_config: Config
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    vpcs = ec2_client.describe_vpcs()
    if vpcs.get('Vpcs', None):
        output_vpc_list(output_media='console', vpcs=vpcs)
    else:
        exit("No VPCs found.")


def vpc_info(aws_config=None, vpc_id=None):
    """
    @type aws_config: Config
    @type vpc_id: unicode
    """
    ec2_client = get_client(client_type='ec2', config=aws_config)
    try:
        vpcs = ec2_client.describe_vpcs(VpcIds=[vpc_id])
        all_subnets = ec2_client.describe_subnets(Filters=[{'Name': 'vpc-id', 'Values': [vpc_id]}])
        output_vpc_info(output_media='console', vpc=vpcs['Vpcs'][0], subnets=all_subnets)
    except (ClientError, IndexError):
        exit("Cannot find VPC: {0}".format(vpc_id))



# -*- coding: utf-8 -*-
from __future__ import (absolute_import, print_function, unicode_literals)
from acli.output import (output_ascii_table, dash_if_none)
from colorclass import Color, Windows
Windows.enable(auto_colors=True, reset_atexit=True)


def get_tag(name=None, tags=None):
    if tags:
        for tag in tags:
            if tag.get('Key') == name:
                return tag.get('Value')


def output_vpc_list(output_media=None, vpcs=None):
    """
    @type output_media: unicode
    @type vpcs: dict
    """
    if output_media == 'console':
        td = list()
        td.append([Color('{autoblue}vpc id{/autoblue}'), Color('{autoblue}name{/autoblue}'),
                   Color('{autoblue}CIDR block{/autoblue}'), Color('{autoblue}tenancy{/autoblue}'),
                   Color('{autoblue}state{/autoblue}'), Color('{autoblue}DHCP options{/autoblue}'),
                   Color('{autoblue}default vpc{/autoblue}')])
        for vpc in vpcs.get('Vpcs'):
            vpcid = vpc.get('VpcId')
            cidr_block = vpc.get('CidrBlock')
            tenancy = vpc.get('InstanceTenancy')
            state = vpc.get('State')
            dhcpoptions = vpc.get('DhcpOptionsId')
            default = str(vpc.get('IsDefault'))
            td.append([vpcid,
                       dash_if_none(get_tag(name='Name', tags=vpc.get('Tags', None))),
                       dash_if_none(cidr_block),
                       dash_if_none(tenancy),
                       dash_if_none(state),
                       dash_if_none(dhcpoptions),
                       default])
        output_ascii_table(table_title=Color('{autowhite}VPCs{/autowhite}'),
                           table_data=td,
                           inner_heading_row_border=True)
    exit(0)


def output_vpc_info(output_media=None, vpc=None, subnets=None):
    """
    @type output_media: unicode
    @type vpc: ec2.Vpc
    """
    if vpc:
        if output_media == 'console':
            td = list()
            td.append([Color('{autoblue}vpc id{/autoblue}'), vpc.get('VpcId')])
            td.append([Color('{autoblue}CIDR block{/autoblue}'), vpc.get('CidrBlock')])
            td.append([Color('{autoblue}default{/autoblue}'), str(vpc.get('IsDefault'))])
            td.append([Color('{autoblue}tenancy{/autoblue}'), vpc.get('InstanceTenancy')])
            td.append([Color('{autoblue}state{/autoblue}'), dash_if_none(vpc.get('State'))])
            td.append([Color('{autoblue}tags{/autoblue}'), " "])
            if vpc.get('Tags'):
                for vpc_tag in vpc.get('Tags'):
                    td.append([Color('{autoblue}'+ "{0}".format(vpc_tag.get('Key'))+'{/autoblue}'),
                               " {0}".format(vpc_tag.get('Value'))])
            if subnets:
                td.append(["{0}".format('-' * 30), "{0}".format('-' * 30)])
                td.append([Color('{autowhite}SUBNETS{/autowhite}'), " "])
                for subnet in subnets.get('Subnets'):
                    td.append(["{0}".format('-' * 30),
                               "{0}".format('-' * 30)])
                    td.append([Color('{autoblue}subnet id{/autoblue}'),
                               subnet.get('SubnetId')])
                    td.append([Color('{autoblue}az{/autoblue}'),
                               subnet.get('AvailabilityZone')])
                    td.append([Color('{autoblue}state{/autoblue}'),
                               subnet.get('State')])
                    td.append([Color('{autoblue}available IPs{/autoblue}'),
                               str(subnet.get('AvailableIpAddressCount'))])
                    td.append([Color('{autoblue}CIDR block{/autoblue}'),
                               subnet.get('CidrBlock')])
                    td.append([Color('{autoblue}default for az{/autoblue}'),
                               str(subnet.get('DefaultForAz'))])
                    td.append([Color('{autoblue}map public ip on launch{/autoblue}'),
                               str(subnet.get('MapPublicIpOnLaunch'))])
                    if subnet.get('Tags'):
                        td.append([Color('{autoblue}tags{/autoblue}'), "-"])
                        for tag in subnet.get('Tags'):
                            tag_key, tag_value = dash_if_none(tag.get('Key', None)), dash_if_none(tag.get('Value', None))
                            td.append([Color('{autoblue}'+" {}".format(tag_key)+'{/autoblue}'), "{}".format(tag_value)])
            output_ascii_table(table_title=Color('{autowhite}vpc info{/autowhite}'),
                               table_data=td)
    else:
        exit('VPC does not exist.')
    exit(0)
